@@ -1,5 +1,6 @@
 import io
 import os
+import re
 import shutil
 import sys
 from mock import patch, Mock, call
@@ -22,7 +23,7 @@ from autoapi.mappers.python import (
 def builder():
     cwd = os.getcwd()
 
-    def build(test_dir, confoverrides=None):
+    def build(test_dir, confoverrides=None, **kwargs):
         os.chdir("tests/python/{0}".format(test_dir))
         app = Sphinx(
             srcdir=".",
@@ -31,6 +32,7 @@ def builder():
             doctreedir="_build/.doctrees",
             buildername="text",
             confoverrides=confoverrides,
+            **kwargs
         )
         app.build(force_all=True)
 
@@ -125,6 +127,9 @@ class TestSimpleStubModule(object):
         with io.open(example_path, encoding="utf8") as example_handle:
             example_file = example_handle.read()
 
+        # Are pyi files preferred
+        assert "DoNotFindThis" not in example_file
+
         assert "class example.Foo" in example_file
         assert "class Meta" in example_file
         assert "Another class var docstring" in example_file
@@ -137,6 +142,22 @@ class TestSimpleStubModule(object):
         assert "Set an attribute" in example_file
 
 
+class TestSimpleStubModuleNotPreferred(object):
+    @pytest.fixture(autouse=True, scope="class")
+    def built(self, builder):
+        builder("pyiexample2")
+
+    def test_integration(self):
+        example_path = "_build/text/autoapi/example/index.txt"
+        with io.open(example_path, encoding="utf8") as example_handle:
+            example_file = example_handle.read()
+
+        # Are py files preferred
+        assert "DoNotFindThis" not in example_file
+
+        assert "Foo" in example_file
+
+
 @pytest.mark.skipif(
     sys.version_info < (3, 6), reason="Annotations are invalid in Python <3.5"
 )
@@ -171,6 +192,32 @@ class TestPy3Module(object):
 
         assert "global_a :A" in example_file
 
+        if sphinx.version_info >= (2, 1):
+            assert "my_method(self) -> str" in example_file
+
+    def test_overload(self):
+        example_path = "_build/text/autoapi/example/index.txt"
+        with io.open(example_path, encoding="utf8") as example_handle:
+            example_file = example_handle.read()
+
+        assert "overloaded_func(a: float" in example_file
+        assert "overloaded_func(a: str" in example_file
+        assert "overloaded_func(a: Union" not in example_file
+        assert "Overloaded function" in example_file
+
+        assert "overloaded_method(self, a: float" in example_file
+        assert "overloaded_method(self, a: str" in example_file
+        assert "overloaded_method(self, a: Union" not in example_file
+        assert "Overloaded method" in example_file
+
+        assert "overloaded_class_method(cls, a: float" in example_file
+        assert "overloaded_class_method(cls, a: str" in example_file
+        assert "overloaded_class_method(cls, a: Union" not in example_file
+        assert "Overloaded method" in example_file
+
+        assert "undoc_overloaded_func" in example_file
+        assert "undoc_overloaded_method" in example_file
+
     def test_async(self):
         example_path = "_build/text/autoapi/example/index.txt"
         with io.open(example_path, encoding="utf8") as example_handle:
@@ -184,6 +231,23 @@ class TestPy3Module(object):
             assert "async_function" in example_file
 
 
+@pytest.mark.skipif(
+    sys.version_info < (3, 6), reason="Annotations are invalid in Python <3.5"
+)
+def test_py3_hiding_undoc_overloaded_members(builder):
+    confoverrides = {"autoapi_options": ["members", "special-members"]}
+    builder("py3example", confoverrides=confoverrides)
+
+    example_path = "_build/text/autoapi/example/index.txt"
+    with io.open(example_path, encoding="utf8") as example_handle:
+        example_file = example_handle.read()
+
+    assert "overloaded_func" in example_file
+    assert "overloaded_method" in example_file
+    assert "undoc_overloaded_func" not in example_file
+    assert "undoc_overloaded_method" not in example_file
+
+
 @pytest.mark.skipif(
     sys.version_info < (3,), reason="Annotations are not supported in astroid<2"
 )
@@ -424,9 +488,9 @@ def test_skipping_members(builder):
 @pytest.mark.parametrize(
     "value,order",
     [
-        ("bysource", ["Foo", "decorator_okay", "Bar"]),
-        ("alphabetical", ["Bar", "Foo", "decorator_okay"]),
-        ("groupwise", ["Bar", "Foo", "decorator_okay"]),
+        ("bysource", [".Foo", ".decorator_okay", ".Bar"]),
+        ("alphabetical", [".Bar", ".Foo", ".decorator_okay"]),
+        ("groupwise", [".Bar", ".Foo", ".decorator_okay"]),
     ],
 )
 def test_order_members(builder, value, order):
@@ -632,6 +696,12 @@ class TestComplexPackage(object):
         assert "unicode_str" in foo_file
 
 
+class TestComplexPackageParallel(object):
+    @pytest.fixture(autouse=True, scope="class")
+    def built(self, builder):
+        builder("pypackagecomplex", parallel=2)
+
+
 @pytest.mark.skipif(
     sys.version_info < (3, 3), reason="Implicit namespace not supported in python < 3.3"
 )
